@@ -707,8 +707,8 @@ class _LxmlFrameParser(_HtmlFrameParser):
         --------
         pandas.io.html._HtmlFrameParser._build_doc
         """
-        from lxml.html import parse, fromstring, HTMLParser
         from lxml.etree import XMLSyntaxError
+        from lxml.html import HTMLParser, fromstring, parse
 
         parser = HTMLParser(recover=True, encoding=self.encoding)
 
